#include "MListaParadas.h"

namespace MListaParadas
{     

    void InsertarAutobuses(TLista &l, TNatural parada, TAutobuses autobuses, TError &err);
    void incrementarCircular(TLista l, TLista &ptr);
    
    void CrearLista(TLista &l)
    {
       l = NULL;
    }
    
    bool ListaVacia(TLista l)
    {
       return (l==NULL);           
    }

	void LeerParadasFichero(TLista &l, TCadena nomFich, TError &err)
    {
         ifstream fich;
         TNatural parada;
         TAutobuses autobuses;
          
         fich.open(nomFich);
         if ( (fich.bad()) || (fich.fail()) )
         {
            err = ErrorAperturaFichero;  
         }
         else
         {
            err = NoError;
            DestruirParadas(l);
            CrearLista(l); 
            fich >> parada; 
            fich.ignore();
            while (!fich.eof())
            {
               LeerAutobusesFich(autobuses,fich);
               InsertarParada(l,parada,err);
               InsertarAutobuses(l,parada,autobuses,err);
               fich >> parada;
               fich.ignore(); 
            }
            fich.close();
         }
    }

    void EscribirParadasFichero(TLista l, TCadena nomFich, TError &err)
    {
         TLista ptr;
         ofstream fich;
         
         fich.open(nomFich);
         if ( (fich.bad()) || (fich.fail()) )
         {
            err = ErrorAperturaFichero;  
         }
         else
         {
            err = NoError;
            ptr = l;
            while (ptr != NULL)
            {
                 fich << ptr->parada.cod << SP;
                 EscribirAutobusesFichero(ptr->parada.autobuses,fich);
                 ptr = ptr->sig;    
            }      
         }  
         fich.close();
    }     
    
    void InsertarParada(TLista &l, TNatural parada, TError &err) 
    {
         TLista nuevo;
                  
         if ( (ListaVacia(l)) || (parada < l->parada.cod) )
         {   // Inserto por la cabeza
             err = NoError;
             nuevo = new (TNodo);
             nuevo->parada.cod = parada;
             nuevo->parada.autobuses.num_buses = 0;
             nuevo->sig = l;
             l = nuevo;
         }
         else if (parada == l->parada.cod)
         {   
             err = ErrorParadaExiste;
         }
         else
         {
             InsertarParada(l->sig,parada,err);
         } 
    }

    void InsertarAutobuses(TLista &l, TNatural parada, TAutobuses autobuses, TError &err)
    {
        
         if ( (ListaVacia(l)) || (parada < l->parada.cod) )
         {
             err = ErrorNoEncontrado;
         }
         else if (parada == l->parada.cod)
         {
             err = NoError;
             l->parada.autobuses = autobuses;
         }
         else
         {
             InsertarAutobuses(l->sig,parada,autobuses,err);
         }
    }     

    void BorrarParada(TLista &l,TNatural parada, TError &err)
    {
        TLista ptr;
         
         if ( (ListaVacia(l)) || (parada < l->parada.cod) )
         {
             err = ErrorNoEncontrado;
         }
         else if (parada == l->parada.cod)
         {
             err = NoError;
             ptr = l;
             l = l->sig;
             delete(ptr);
         }
         else
         {
             BorrarParada(l->sig,parada,err);
         }
    }
    
    void DestruirParadas(TLista &l)
    {
       TLista aux;
       
       while (l!=NULL)
       {
             aux = l;
             l = l->sig;
             delete(aux);
       }         
    }
    
	void MostrarParadas(TLista l)
    {
       TLista ptr;
       
       ptr = l;
       
       while (ptr!=NULL)
       {
             cout << ptr->parada.cod << SP;
             MostrarAutobuses(ptr->parada.autobuses);
             cout << endl;
             ptr = ptr->sig;
       }

    }

    void SubirBajarPasajeros(TLista &l, TNatural parada, TNatural linea, TNatural numSuben, 
                             TNatural &numRealS, TNatural numBajan, TNatural &numRealB, 
                             TError &err)
    {
        bool ok;
        
        if ( (ListaVacia(l)) || (parada < l->parada.cod) )
         {
             err = ErrorNoEncontrado;
         }
         else if (parada == l->parada.cod)
         {
             err = NoError;
             BajarPasajeros(l->parada.autobuses,linea,numBajan,numRealB,ok);
             if (!ok)
             {
                err = ErrorLineaNoExiste;
             }
             else
             {  
                 SubirPasajeros(l->parada.autobuses,linea,numSuben,numRealS,ok);
             }    
         }
         else
         {
             SubirBajarPasajeros(l->sig,parada,linea,numSuben,numRealS, numBajan, numRealB, err);
         }
    }     
    
    void FueraDeServicio(TLista &l, TNatural parada, TError &err)
    {
         TLista ptr, ptr_borrar;
         TAutobuses a;
         TAutobus bus;
         TNatural i;
         bool ok,ok_ext;

         ptr = l;             
         while ((ptr != NULL) && (parada > ptr->parada.cod))
         {
               ptr = ptr -> sig;
         }
                           
         if ( (ptr != NULL) && (parada == ptr->parada.cod) )
         {
             err = NoError;
             a = ptr->parada.autobuses;
             ptr_borrar = ptr;
             incrementarCircular(l,ptr);
             ExtraerAutobus(a,bus,ok_ext);
             while ((ptr != ptr_borrar) && (ok_ext))
             {
                 AniadirAutobus(ptr->parada.autobuses,bus,ok);
                 if (!ok)
                 {
                    incrementarCircular(l,ptr);
                 }         
                 else
                 {
                     ExtraerAutobus(a,bus,ok_ext);
                 }    
             }
             if (ok_ext) //Ha salido porque ptr == ptr_borrar
             {
                   err = ErrorNoHayEspacio; 
             }
             else
             {
                   BorrarParada(l,parada,err);      
             }      
         }    
         else
         {
             err = ErrorNoEncontrado;    
         }    
    }     

    void incrementarCircular(TLista l, TLista &ptr)
    {
         ptr = ptr -> sig;
         if (ptr == NULL)
         {
            ptr = l;     
         }   
    }     
}

